@@ -36,16 +36,18 @@ import (
 	"github.com/snapcore/snapd/bootloader/bootloadertest"
 	"github.com/snapcore/snapd/cmd/snaplock/runinhibit"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/overlord/snapstate/backend"
 	"github.com/snapcore/snapd/progress"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/quota"
 	"github.com/snapcore/snapd/snap/snaptest"
 	"github.com/snapcore/snapd/systemd"
 	"github.com/snapcore/snapd/testutil"
 	"github.com/snapcore/snapd/timings"
-
-	"github.com/snapcore/snapd/overlord/snapstate/backend"
+	"github.com/snapcore/snapd/wrappers"
 )
 
 type linkSuiteCommon struct {
@@ -58,6 +60,7 @@ type linkSuiteCommon struct {
 
 func (s *linkSuiteCommon) SetUpTest(c *C) {
 	s.BaseTest.SetUpTest(c)
+
 	dirs.SetRootDir(c.MkDir())
 	s.AddCleanup(func() { dirs.SetRootDir("") })
 
@@ -155,7 +158,7 @@ version: 1.0
 	reboot, err := s.be.LinkSnap(info, mockDev, backend.LinkContext{}, s.perfTimings)
 	c.Assert(err, IsNil)
 
-	c.Check(reboot, Equals, false)
+	c.Check(reboot, Equals, boot.RebootInfo{RebootRequired: false})
 
 	mountDir := info.MountDir()
 	dataDir := info.DataDir()
@@ -194,7 +197,59 @@ type: base
 
 	reboot, err := s.be.LinkSnap(info, coreDev, backend.LinkContext{}, s.perfTimings)
 	c.Assert(err, IsNil)
-	c.Check(reboot, Equals, true)
+	c.Check(reboot, Equals, boot.RebootInfo{RebootRequired: true})
+}
+
+func (s *linkSuite) TestLinkNoSetNextBootWhenPreseeding(c *C) {
+	restore := release.MockOnClassic(false)
+	defer restore()
+
+	be := backend.NewForPreseedMode()
+	coreDev := boottest.MockUC20Device("run", nil)
+
+	bl := boottest.MockUC20RunBootenv(bootloadertest.Mock("mock", c.MkDir()))
+	bootloader.Force(bl)
+	defer bootloader.Force(nil)
+
+	const yaml = `name: pc-kernel
+version: 1.0
+type: kernel
+`
+	info := snaptest.MockSnap(c, yaml, &snap.SideInfo{Revision: snap.R(11)})
+
+	reboot, err := be.LinkSnap(info, coreDev, backend.LinkContext{}, s.perfTimings)
+	c.Assert(err, IsNil)
+	c.Check(reboot, DeepEquals, boot.RebootInfo{})
+}
+
+func (s *linkSuite) TestLinkSnapdSnapCallsWrappersWithPreseedingFlag(c *C) {
+	restore := release.MockOnClassic(false)
+	defer restore()
+
+	var called bool
+	restoreAddSnapdSnapWrappers := backend.MockWrappersAddSnapdSnapServices(func(s *snap.Info, opts *wrappers.AddSnapdSnapServicesOptions, inter wrappers.Interacter) error {
+		c.Check(opts.Preseeding, Equals, true)
+		called = true
+		return nil
+	})
+	defer restoreAddSnapdSnapWrappers()
+
+	be := backend.NewForPreseedMode()
+	coreDev := boottest.MockUC20Device("run", nil)
+
+	bl := boottest.MockUC20RunBootenv(bootloadertest.Mock("mock", c.MkDir()))
+	bootloader.Force(bl)
+	defer bootloader.Force(nil)
+
+	const yaml = `name: snapd
+version: 1.0
+type: snapd
+`
+	info := snaptest.MockSnap(c, yaml, &snap.SideInfo{Revision: snap.R(11)})
+
+	_, err := be.LinkSnap(info, coreDev, backend.LinkContext{}, s.perfTimings)
+	c.Assert(err, IsNil)
+	c.Assert(called, Equals, true)
 }
 
 func (s *linkSuite) TestLinkDoIdempotent(c *C) {
@@ -304,7 +359,13 @@ type: snapd
 		{"usr/lib/systemd/user/snapd.session-agent.service", "[Unit]\n[Service]\nExecStart=/usr/bin/snap session-agent"},
 		{"usr/lib/systemd/user/snapd.session-agent.socket", "[Unit]\n[Socket]\nListenStream=%t/snap-session.socket"},
 	}
-	info := snaptest.MockSnapWithFiles(c, yaml, &snap.SideInfo{Revision: snap.R(11)}, snapdUnits)
+	otherFiles := [][]string{
+		// D-Bus activation files
+		{"usr/share/dbus-1/services/io.snapcraft.Launcher.service", "[D-BUS Service]\nName=io.snapcraft.Launcher"},
+		{"usr/share/dbus-1/services/io.snapcraft.Settings.service", "[D-BUS Service]\nName=io.snapcraft.Settings"},
+		{"usr/share/dbus-1/services/io.snapcraft.SessionAgent.service", "[D-BUS Service]\nName=io.snapcraft.SessionAgent"},
+	}
+	info := snaptest.MockSnapWithFiles(c, yaml, &snap.SideInfo{Revision: snap.R(11)}, append(snapdUnits, otherFiles...))
 	return info, snapdUnits
 }
 
@@ -321,7 +382,7 @@ func (s *linkSuite) TestLinkSnapdSnapOnCore(c *C) {
 
 	reboot, err := s.be.LinkSnap(info, mockDev, backend.LinkContext{}, s.perfTimings)
 	c.Assert(err, IsNil)
-	c.Assert(reboot, Equals, false)
+	c.Assert(reboot, Equals, boot.RebootInfo{RebootRequired: false})
 
 	// system services
 	c.Check(filepath.Join(dirs.SnapServicesDir, "snapd.service"), testutil.FileContains,
@@ -350,6 +411,13 @@ Options=bind
 WantedBy=snapd.service
 `, info.MountDir())
 	c.Check(filepath.Join(dirs.SnapServicesDir, "usr-lib-snapd.mount"), testutil.FileEquals, mountUnit)
+	// D-Bus service activation files for snap userd
+	c.Check(filepath.Join(dirs.SnapDBusSessionServicesDir, "io.snapcraft.Launcher.service"), testutil.FileEquals,
+		"[D-BUS Service]\nName=io.snapcraft.Launcher")
+	c.Check(filepath.Join(dirs.SnapDBusSessionServicesDir, "io.snapcraft.Settings.service"), testutil.FileEquals,
+		"[D-BUS Service]\nName=io.snapcraft.Settings")
+	c.Check(filepath.Join(dirs.SnapDBusSessionServicesDir, "io.snapcraft.SessionAgent.service"), testutil.FileEquals,
+		"[D-BUS Service]\nName=io.snapcraft.SessionAgent")
 }
 
 type linkCleanupSuite struct {
@@ -407,7 +475,7 @@ Icon=${SNAP}/bin.png
 Exec=bin
 `), 0644), IsNil)
 
-	// sanity checks
+	// validity checks
 	for _, d := range []string{dirs.SnapBinariesDir, dirs.SnapDesktopFilesDir, dirs.SnapServicesDir, dirs.SnapDBusSystemServicesDir, dirs.SnapDBusSessionServicesDir} {
 		os.MkdirAll(d, 0755)
 		l, err := filepath.Glob(filepath.Join(d, "*"))
@@ -476,7 +544,7 @@ func (s *linkCleanupSuite) TestLinkCleanupOnSystemctlFail(c *C) {
 }
 
 func (s *linkCleanupSuite) TestLinkCleansUpDataDirAndSymlinksOnSymlinkFail(c *C) {
-	// sanity check
+	// validity check
 	c.Assert(s.info.DataDir(), testutil.FileAbsent)
 
 	// the mountdir symlink is currently the last thing in LinkSnap that can
@@ -496,7 +564,7 @@ func (s *linkCleanupSuite) TestLinkCleansUpDataDirAndSymlinksOnSymlinkFail(c *C)
 func (s *linkCleanupSuite) TestLinkRunsUpdateFontconfigCachesClassic(c *C) {
 	current := filepath.Join(s.info.MountDir(), "..", "current")
 
-	for _, dev := range []boot.Device{mockDev, mockClassicDev} {
+	for _, dev := range []snap.Device{mockDev, mockClassicDev} {
 		var updateFontconfigCaches int
 		restore := backend.MockUpdateFontconfigCaches(func() error {
 			c.Assert(osutil.FileExists(current), Equals, false)
@@ -590,7 +658,7 @@ func (s *linkCleanupSuite) testLinkCleanupFailedSnapdSnapOnCorePastWrappers(c *C
 	}
 	reboot, err := s.be.LinkSnap(info, mockDev, linkCtx, s.perfTimings)
 	c.Assert(err, ErrorMatches, fmt.Sprintf("symlink %s /.*/snapd/current: permission denied", info.Revision))
-	c.Assert(reboot, Equals, false)
+	c.Assert(reboot, Equals, boot.RebootInfo{RebootRequired: false})
 
 	checker := testutil.FilePresent
 	if firstInstall {
@@ -606,6 +674,11 @@ func (s *linkCleanupSuite) testLinkCleanupFailedSnapdSnapOnCorePastWrappers(c *C
 	c.Check(filepath.Join(dirs.SnapUserServicesDir, "snapd.session-agent.socket"), checker)
 	c.Check(filepath.Join(dirs.SnapServicesDir, "usr-lib-snapd.mount"), checker)
 	c.Check(filepath.Join(runinhibit.InhibitDir, "snapd.lock"), testutil.FileAbsent)
+
+	// D-Bus service activation
+	c.Check(filepath.Join(dirs.SnapDBusSessionServicesDir, "io.snapcraft.Launcher.service"), checker)
+	c.Check(filepath.Join(dirs.SnapDBusSessionServicesDir, "io.snapcraft.Settings.service"), checker)
+	c.Check(filepath.Join(dirs.SnapDBusSessionServicesDir, "io.snapcraft.SessionAgent.service"), checker)
 }
 
 func (s *linkCleanupSuite) TestLinkCleanupFailedSnapdSnapFirstInstallOnCore(c *C) {
@@ -657,9 +730,9 @@ func (s *snapdOnCoreUnlinkSuite) TestUndoGeneratedWrappers(c *C) {
 
 	reboot, err := s.be.LinkSnap(info, mockDev, backend.LinkContext{}, s.perfTimings)
 	c.Assert(err, IsNil)
-	c.Assert(reboot, Equals, false)
+	c.Assert(reboot, Equals, boot.RebootInfo{RebootRequired: false})
 
-	// sanity checks
+	// validity checks
 	c.Check(filepath.Join(dirs.SnapServicesDir, "snapd.service"), testutil.FileContains,
 		fmt.Sprintf("[Service]\nExecStart=%s/usr/lib/snapd/snapd\n", info.MountDir()))
 	// expecting all generated untis to be present
@@ -742,7 +815,7 @@ apps:
 	_, err := s.be.LinkSnap(info, mockDev, linkCtxWithTooling, s.perfTimings)
 	c.Assert(err, IsNil)
 	c.Assert(filepath.Join(dirs.SnapServicesDir, "snap.hello.svc.service"), testutil.FileContains,
-		`Requires=usr-lib-snapd.mount
+		`Wants=usr-lib-snapd.mount
 After=usr-lib-snapd.mount`)
 
 	// remove it now
@@ -757,3 +830,28 @@ After=usr-lib-snapd.mount`)
 	c.Assert(err, IsNil)
 	c.Assert(filepath.Join(dirs.SnapServicesDir, "snap.hello.svc.service"), Not(testutil.FileContains), `usr-lib-snapd.mount`)
 }
+
+func (s *linkSuite) TestLinkOptHasQuotaGroup(c *C) {
+	const yaml = `name: hello
+version: 1.0
+
+apps:
+ svc:
+   command: svc
+   daemon: simple
+`
+	info := snaptest.MockSnap(c, yaml, &snap.SideInfo{Revision: snap.R(11)})
+
+	grp, err := quota.NewGroup("foogroup", quota.NewResourcesBuilder().WithMemoryLimit(quantity.SizeMiB).Build())
+	c.Assert(err, IsNil)
+
+	linkCtxWithGroup := backend.LinkContext{
+		ServiceOptions: &wrappers.SnapServiceOptions{
+			QuotaGroup: grp,
+		},
+	}
+	_, err = s.be.LinkSnap(info, mockDev, linkCtxWithGroup, s.perfTimings)
+	c.Assert(err, IsNil)
+	c.Assert(filepath.Join(dirs.SnapServicesDir, "snap.hello.svc.service"), testutil.FileContains,
+		"\nSlice=snap.foogroup.slice\n")
+}
